from datetime import datetime
from pathlib import Path
from typing import Dict

from croniter import croniter
from croniter.croniter import CroniterBadCronError


def transform_minutes(minutes: str) -> str:
    return f"*/{minutes} * * * *"


def transform_hours(hours: str) -> str:
    return f"0 */{hours} * * *"


def validate_cron_expr(expr: str, path: Path) -> None:
    base = datetime(2000, 1, 1)
    try:
        croniter(expr, base)
    except CroniterBadCronError as e:
        raise ValueError(f"Malformed file, {path} \n {e}")


def cron_update(config: Dict) -> None:
    schedule = config["schedule"]
    if "cronExpression" in schedule:
        cron = schedule["cronExpression"]
    elif "runEveryHour" in schedule:
        cron = transform_hours(schedule["runEveryHour"])
    elif "runEveryMinute" in schedule:
        cron = transform_minutes(schedule["runEveryMinute"])
    else:
        raise ValueError(
            "Expected the schedule section of the configuration file to contain one of the following:"
            " 'cronExpression', 'runEveryHour', 'runEveryMin'"
        )

    config["schedule"].update({"cronExpression": cron})
    if "runEveryHour" in schedule:
        config["schedule"].pop("runEveryHour")
    if "runEveryMinute" in schedule:
        config["schedule"].pop("runEveryMinute")


def validate_and_update_cron(config: Dict, path: Path) -> None:
    if "cronExpression" in config["schedule"]:
        validate_cron_expr(config["schedule"]["cronExpression"], path)

    cron_update(config)



from pathlib import Path
from typing import List


class Deployment:
    def __init__(
        self,
        project: str,
        code_path: Path,
        version: str,
        secret_names: List[str],
    ):
        self.project = project
        self.code_path = code_path
        self.version = version
        self.secret_names = secret_names



from pathlib import Path
from typing import Dict, List

from cerberus import Validator
from ruamel.yaml import YAML

import cognite.airworkflow.util.convert_string_times as window_size
import cognite.airworkflow.util.cron as cron
from cognite.airworkflow.model.config import Config
from cognite.airworkflow.util import env

working_path = env.get_env_value("PWD")
ROOT_DIR = Path(working_path)
REPOCONFIG_PATH = ROOT_DIR / Path("repoconfig.yaml")


def write_yaml(path: Path, content: Dict, *, create_dirs: bool = False) -> None:
    if create_dirs:
        path.parent.mkdir(exist_ok=True)
    YAML(typ="safe").dump(content, path)


def load_yaml(path: Path) -> Dict:
    yaml = YAML(typ="safe").load(path)
    assert isinstance(yaml, dict)
    return yaml


def get_validator(schema_path: Path) -> Validator:
    schema = load_yaml(schema_path)
    validator = Validator(schema)
    return validator


def load_and_validate(path: Path, schema_path: Path, function_name: str) -> Config:
    yaml = load_yaml(path)
    repoconfig = load_yaml(REPOCONFIG_PATH)
    validator = get_validator(schema_path)
    is_valid = validator.validate(yaml)
    if not is_valid:
        raise ValueError(f"Malformed file, {path} \n {validator.errors}")

    if "schedule" in yaml:
        cron.validate_and_update_cron(yaml, path)
        window_size.window_size_update(yaml)

    return Config(yaml, function_name, repoconfig)


def load_and_validate_multiple(paths: List[Path], schema_path: Path) -> Dict[Path, Config]:
    return {p.parent.parent: load_and_validate(p, schema_path, p.parent.parent.parts[-1]) for p in paths}



# import os
from pathlib import Path
from typing import Dict, Set

from ruamel.yaml import YAML

# import cognite.airworkflow.util.file as file
from cognite.airworkflow.util import env


def load_yaml(path: Path) -> Dict:
    yaml = YAML(typ="safe").load(path)
    assert isinstance(yaml, dict)
    return yaml


working_path = env.get_env_value("PWD")
ROOT_DIR = Path(working_path)
FUNCTIONS_PATH = ROOT_DIR / "functions"
IGNORE_MODELS_PATH = ROOT_DIR / ".ignore_models"

# Paths within a functions folder
FUNCTION_REL_PATH = Path("function")
REPOCONFIG_PATH = Path("repoconfig.yaml")
FUNCTION_REL_CONFIG_PATH = FUNCTION_REL_PATH / "config.yaml"


def project_name_finder() -> Set:
    repo_config = load_yaml(REPOCONFIG_PATH)["Projectproperties"].keys()
    return set(repo_config)
